from __future__ import annotations

from .base import TelegramObject


class ForumTopicClosed(TelegramObject):
    """
    This object represents a service message about a forum topic closed in the chat. Currently holds no information.

    Source: https://core.telegram.org/bots/api#forumtopicclosed
    """
