# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiogram',
 'aiogram.client',
 'aiogram.client.session',
 'aiogram.client.session.middlewares',
 'aiogram.dispatcher',
 'aiogram.dispatcher.event',
 'aiogram.dispatcher.middlewares',
 'aiogram.filters',
 'aiogram.fsm',
 'aiogram.fsm.storage',
 'aiogram.handlers',
 'aiogram.methods',
 'aiogram.types',
 'aiogram.utils',
 'aiogram.utils.i18n',
 'aiogram.webhook']

package_data = \
{'': ['*']}

install_requires = \
['aiofiles>=22.1.0,<23.0.0',
 'aiohttp>=3.8.3,<4.0.0',
 'certifi>=2022.9.24,<2023.0.0',
 'magic-filter>=1.0.9,<2.0.0',
 'pydantic>=1.10.2,<2.0.0']

extras_require = \
{'fast:sys_platform == "darwin" or sys_platform == "linux"': ['uvloop>=0.17.0,<0.18.0'],
 'i18n': ['Babel>=2.9.1,<3.0.0'],
 'proxy': ['aiohttp-socks>=0.7.1,<0.8.0'],
 'redis': ['redis>=4.3.4,<5.0.0']}

setup_kwargs = {
    'name': 'aiogram',
    'version': '3.0.0b6',
    'description': 'Modern and fully asynchronous framework for Telegram Bot API',
    'long_description': '####################\naiogram |beta badge|\n####################\n\n.. danger::\n    This version is still in development!\n\n.. image:: https://img.shields.io/pypi/l/aiogram.svg\n    :target: https://opensource.org/licenses/MIT\n    :alt: MIT License\n\n.. image:: https://img.shields.io/pypi/pyversions/aiogram.svg\n    :target: https://pypi.python.org/pypi/aiogram\n    :alt: Supported python versions\n\n.. image:: https://img.shields.io/badge/Telegram%20Bot%20API-6.3-blue.svg?logo=telegram\n    :target: https://core.telegram.org/bots/api\n    :alt: Telegram Bot API\n\n.. image:: https://github.com/aiogram/aiogram/workflows/Tests/badge.svg?branch=dev-3.x\n    :target: https://github.com/aiogram/aiogram/actions\n    :alt: Tests\n\n.. image:: https://img.shields.io/pypi/v/aiogram.svg\n    :target: https://pypi.python.org/pypi/aiogram\n    :alt: PyPi Package Version\n\n.. image:: https://img.shields.io/pypi/status/aiogram.svg\n    :target: https://pypi.python.org/pypi/aiogram\n    :alt: PyPi status\n\n.. image:: https://img.shields.io/pypi/dm/aiogram.svg\n    :target: https://pypi.python.org/pypi/aiogram\n    :alt: Downloads\n\n.. image:: https://img.shields.io/badge/telegram-aiogram-blue.svg\n    :target: https://t.me/aiogram_live\n    :alt: [Telegram] aiogram live\n\n.. image:: https://img.shields.io/codecov/c/github/aiogram/aiogram?style=flat-square\n    :target: https://app.codecov.io/gh/aiogram/aiogram\n    :alt: Codecov\n\n**aiogram** is a modern and fully asynchronous framework for\n`Telegram Bot API <https://core.telegram.org/bots/api>`_ written in Python 3.8 using\n`asyncio <https://docs.python.org/3/library/asyncio.html>`_ and\n`aiohttp <https://github.com/aio-libs/aiohttp>`_.\n\nMake your bots faster and more powerful!\n\nDocumentation:\n - 🇺🇸 `English <https://docs.aiogram.dev/en/dev-3.x/>`_\n - 🇺🇦 `Ukrainian <https://docs.aiogram.dev/uk_UA/dev-3.x/>`_\n\n\n.. danger::\n\n    **Breaking News:**\n\n    *aiogram* 3.0 has breaking changes.\n\n    It breaks backward compatibility by introducing new breaking changes!\n\nFeatures\n========\n\n- Asynchronous (`asyncio docs <https://docs.python.org/3/library/asyncio.html>`_, :pep:`492`)\n- Has type hints (:pep:`484`) and can be used with `mypy <http://mypy-lang.org/>`_\n- Supports `PyPy <https://www.pypy.org/>`_\n- Supports `Telegram Bot API 6.3 <https://core.telegram.org/bots/api>`_ and gets fast updates to the latest versions of the Bot API\n- Telegram Bot API integration code was `autogenerated <https://github.com/aiogram/tg-codegen>`_ and can be easily re-generated when API gets updated\n- Updates router (Blueprints)\n- Has Finite State Machine\n- Uses powerful `magic filters <https://docs.aiogram.dev/en/dev-3.x/dispatcher/filters/magic_filters.html#magic-filters>`\n- Middlewares (incoming updates and API calls)\n- Provides `Replies into Webhook <https://core.telegram.org/bots/faq#how-can-i-make-requests-in-response-to-updates>`_\n- Integrated I18n/L10n support with GNU Gettext (or Fluent)\n\n\n.. warning::\n\n    It is strongly advised that you have prior experience working\n    with `asyncio <https://docs.python.org/3/library/asyncio.html>`_\n    before beginning to use **aiogram**.\n\n    If you have any questions, you can visit our community chats on Telegram:\n\n    - 🇺🇸 `@aiogram <https://t.me/aiogram>`_\n    - 🇺🇦 `@aiogramua <https://t.me/aiogramua>`_\n    - 🇺🇿 `@aiogram_uz <https://t.me/aiogram_uz>`_\n    - 🇰🇿 `@aiogram_kz <https://t.me/aiogram_kz>`_\n    - 🇷🇺 `@aiogram_ru <https://t.me/aiogram_ru>`_\n    - 🇮🇷 `@aiogram_fa <https://t.me/aiogram_fa>`_\n    - 🇮🇹 `@aiogram_it <https://t.me/aiogram_it>`_\n    - 🇧🇷 `@aiogram_br <https://t.me/aiogram_br>`_\n\n\n.. |beta badge| image:: https://img.shields.io/badge/-beta-orange\n  :alt: Beta badge\n',
    'author': 'Alex Root Junior',
    'author_email': 'jroot.junior@gmail.com',
    'maintainer': 'Alex Root Junior',
    'maintainer_email': 'jroot.junior@gmail.com',
    'url': 'https://aiogram.dev/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
